@@ -87,11 +87,11 @@ getEstimator <- function(object, estimator="gamma", Pmax=0){
     if( estimator == "logP" ){
       logP <- t( as.matrix( read.table(object$output$logP) ) )
       model_size <- as.matrix( read.table(object$output$model_size) )
-      ncol_Y <- ncol(read.table(object$output$G))
+      ncol_Y <- ncol(read.table(object$output$gamma))
       nIter <- object$input$nIter
       
       covariancePrior <- object$input$covariancePrior
-      if(covariancePrior=="HIW" & file.exists(object$output$Gvisit)){
+      if(covariancePrior=="HIW" & is.null(object$output$Gvisit)){
         Gvisit <- as.matrix( read.table(object$output$Gvisit) )
         ret <- list(logP=logP, model_size=model_size, Gvisit=Gvisit, 
                     ncol_Y=ncol_Y, nIter=nIter, covariancePrior=covariancePrior)
@@ -129,13 +129,13 @@ getEstimator <- function(object, estimator="gamma", Pmax=0){
       | (length(estimator)==2 & sum(estimator %in% c("beta", "gamma"))==2)){
     beta <- as.matrix( read.table(object$output$beta) )
     gamma <- as.matrix( read.table(object$output$gamma) )
-    colnames(beta) <- colnames(gamma) <- names(read.table(object$output$Y,header=T))
-    rownames(gamma) <- names(read.table(object$output$X,header=T))
+    #colnames(beta) <- colnames(gamma) <- names(read.table(object$output$Y,header=T))
+    #rownames(gamma) <- names(read.table(object$output$X,header=T))
     nonpen <- nrow(beta) - nrow(gamma)
     if(nonpen > 0){
       rownames(beta) <- c(names(read.table(object$output$X0,header=T)), names(read.table(object$output$X,header=T)))
     }else{
-      rownames(beta) <- names(read.table(object$output$X,header=T))
+      #rownames(beta) <- names(read.table(object$output$X,header=T))
     }
     covariancePrior <- object$input$covariancePrior
     if( (covariancePrior == "HIW") & ("Gy" %in% estimator) ){
